import { METADATA } from "./constants";
import { DataType } from "./DataType";
export const isSubClass = (derivedClass, baseClass) => {
    return derivedClass === baseClass || derivedClass.prototype instanceof baseClass;
};
const getDataType = (t) => {
    switch (t?.name) {
        case "Boolean":
            return DataType.BOOLEAN;
        case "Number":
            return DataType.NUMBER;
        case "String":
            return DataType.STRING;
        case "Array":
            return DataType.ARRAY;
        default:
            return DataType.OBJECT;
    }
};
const getReflectedDataType = (prototype, propertyKey) => {
    const t = Reflect.getMetadata("design:type", prototype, propertyKey);
    return { type: t, dataType: getDataType(t) };
};
export const initObjectMetadata = (prototype) => {
    if (prototype === Object.prototype) {
        return {};
    }
    if (!prototype.hasOwnProperty(METADATA)) {
        const superPrototype = initObjectMetadata(Object.getPrototypeOf(prototype));
        prototype[METADATA] = {
            ...superPrototype,
            id: getTableName(prototype.constructor),
            fields: superPrototype.fields ? superPrototype.fields.map((f) => ({ ...f })) : [],
        };
    }
    return prototype[METADATA];
};
export const initFieldMetadata = (prototype, propertyKey) => {
    const meta = initObjectMetadata(prototype);
    const fieldIndex = meta.fields.findIndex((f) => f.name === propertyKey);
    if (fieldIndex < 0) {
        const { type, dataType } = getReflectedDataType(prototype, propertyKey);
        const field = {
            name: propertyKey,
            dataType,
            description: "",
            elementDto: dataType === DataType.OBJECT ? getObjectMetadata(type) : undefined,
        };
        meta.fields.push(field);
        return field;
    }
    else {
        //-- shadow clone the field
        const field = { ...meta.fields[fieldIndex] };
        meta.fields[fieldIndex] = field;
        return field;
    }
};
export const getObjectMetadata = (obj) => obj?.prototype?.[METADATA];
export const setObjectMetadata = (obj, metadata) => {
    obj.prototype[METADATA] = metadata;
};
export const getFieldMetadata = (cls, key) => {
    const clsMeta = getObjectMetadata(cls);
    return clsMeta?.fields.find((f) => f.name === key);
};
export const getTableName = (model) => model?.name.toLowerCase();
